-------------------------------------------------------------------------------
--|> Player Data Manager
-- Functions that help acquire and manage data involving the PlayerDB
-------------------------------------------------------------------------------
--|> UPVALUE GLOBALS
-------------------------------------------------------------------------------
local weizPVP = weizPVP
local select, tonumber = select, tonumber
local GetPlayerInfoByGUID = GetPlayerInfoByGUID
local UnitIsEnemy = UnitIsEnemy
local UnitIsPlayer = UnitIsPlayer
local GetUnitName = GetUnitName
local GetGuildInfo = GetGuildInfo
local UnitClass = UnitClass
local UnitRace = UnitRace
local UnitLevel = UnitLevel
local UnitIsDeadOrGhost = UnitIsDeadOrGhost
local GetTime = GetTime
local UnitHealth = UnitHealth
local UnitHealthMax = UnitHealthMax
local strsub = strsub

-------------------------------------------------------------------------------
--|> GLOBAL FUNCTIONS
-------------------------------------------------------------------------------

--=> UpdatePlayerActiveCache
-->  Updates data in the PlayerActiveCache and then calls
--> Sends data off the Lists to be processed
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerActiveCache(name, timestamp, newDataFound, health, stealth, dead, role)
    local playerDBData = weizPVP.PlayerDB[name] or {}
    local newPlayerOnList = true
    local playerCacheData = {}
    if weizPVP.PlayerActiveCache[name] then
        newPlayerOnList = false
        playerCacheData = weizPVP.PlayerActiveCache[name]
    end

    --> Update PlayerCache info
    if playerDBData then
        --> LEVEL
        playerCacheData.Estimated = playerDBData.Estimated
        if playerDBData.Level ~= nil then
            playerCacheData.Level = playerDBData.Level
        else
            playerCacheData.Level = 0
        end

        --> CLASS
        if playerDBData.Class ~= nil then
            playerCacheData.Class = playerDBData.Class
        else
            playerCacheData.Class = "UNKNOWN"
        end

        --> GUILD
        if playerDBData.Guild ~= nil then
            playerCacheData.Guild = playerDBData.Guild
        else
            if playerCacheData.Estimated == true then
                playerCacheData.Guild = "?"
            else
                playerCacheData.Guild = ""
            end
        end

        --> HEALTH
        if health ~= nil then
            playerCacheData.Health = health
        elseif playerCacheData.Health == nil then
            playerCacheData.Health = 100
        end

        --> STEALTH
        if stealth ~= nil then
            playerCacheData.Stealth = stealth
        end

        --> DEAD
        if dead ~= nil then
            playerCacheData.Dead = dead
            if dead == true then
                playerCacheData.Health = 0
            end
        end

        --> ROLE
        if role ~= nil then
            playerCacheData.Role = role
        elseif playerDBData.Role ~= nil then
            playerCacheData.Role = playerDBData.Role
        end

        --> Update PlayerActiveCache Data
        weizPVP.PlayerActiveCache[name] = playerCacheData
        --> Add player Data to lists
        weizPVP:AddPlayerDataToNearby(name, newPlayerOnList)
    end
end

--=> AddNewPlayerDB
--> Adds ONLY NEW Players to DB
-----------------------------------------------------------------------
function weizPVP:AddNewPlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
    local info = {}
    info.Name = nameAndRealm
    info.Class = class or ""
    info.Level = level or 0
    info.Race = race or ""
    info.Role = role or nil
    if estimated == true and guild == nil then
        info.Guild = ""
    else
        info.Guild = guild
    end

    info.Estimated = estimated or true
    weizPVP.PlayerDB[nameAndRealm] = info

    return true
end

--=> Update Player DB
--> All new entries and updates to weizPVP.PlayerDB happen here.
-----------------------------------------------------------------------
function weizPVP:UpdatePlayerDB(nameAndRealm, class, level, race, guild, estimated, dataUpdated, role)
    local dataAdded

    local playerData = weizPVP.PlayerDB[nameAndRealm]

    if not playerData then --> NEW PLAYER
        dataAdded = weizPVP:AddNewPlayerDB(nameAndRealm, class, level, race, guild, estimated, role)
    else --> EXISTING PLAYER
        if playerData.Estimated == true then
            if playerData.Level and level and playerData.Level < level then
                playerData.Level = level
            end

            if estimated == false then
                playerData.Estimated = estimated
            end
            if role ~= nil then
                playerData.Role = role
            end
        else
            --> not estimating, use the level from the db, if higher
            if playerData and level and playerData.Level < level then
                playerData.Level = level
            end
            if role ~= nil then
                playerData.Role = role
            end
        end

        if role ~= nil then
            playerData.Role = role
        end

        if guild ~= nil then
            playerData.Guild = guild
        end

        weizPVP.PlayerDB[nameAndRealm] = playerData
        dataAdded = true
    end
    return dataAdded
end

--=> GET UNIT DATA
-----------------------------------------------------------------------
--> Gets data for events with a UNITID, such as 'target' or 'nameplate'
function weizPVP:GetUnitData(unit)
    --> Confirm Unit is a player that is an enemy or attackable
    if (UnitIsEnemy("player", unit) or weizPVP.Options.GLOBAL_DEBUG) and UnitIsPlayer(unit) then
        local info = {}
        info.Name = GetUnitName(unit, true)
        if info.Name ~= "Unknown" then
            --> Info for DB
            info.Estimated = false
            info.Guild = GetGuildInfo(unit)
            info.Class = select(2, UnitClass(unit))
            local Role = weizPVP:RoleAssign(info.Class)
            if weizPVP.PlayerDB[info.Name] and weizPVP.PlayerDB[info.Name].Role and Role == nil then
                info.Role = weizPVP.PlayerDB[info.Name].Role
            else
                info.Role = Role
            end
            info.Race = select(1, UnitRace(unit))
            info.Level = tonumber(UnitLevel(unit))
            info.Timestamp = GetTime()
            weizPVP.PlayerDB[info.Name] = info

            --> Info for Cache
            local Dead = UnitIsDeadOrGhost(unit)
            local Stealth = false
            local Health = tonumber(UnitHealth(unit) / UnitHealthMax(unit) * 100)
            weizPVP:UpdatePlayerActiveCache(info.Name, info.Timestamp, true, Health, Stealth, Dead, Role)
            return true
        end
    end
    return false
end

--=> GET LEVEL DATA
-------------------------------------------------------------------------------
function weizPVP:GetLevelData(parseData, event, player, playerGUID, spellId, spellName)
    local newDataAdded = false
    if player then
        --> Get basic info from GUID
        local level
        local estimated = true
        local class
        local role
        local race
        local playerData = weizPVP.PlayerDB[player]

        --> NEW PLAYER
        if not playerData then
            newDataAdded = true
            class = select(2, GetPlayerInfoByGUID(playerGUID))
            race = select(4, GetPlayerInfoByGUID(playerGUID))
            role = weizPVP:RoleAssign(class) or nil
        end

        --> If we should be parsing data
        if parseData then
            --> Parse level only for abilities
            local abilityType = strsub(event, 1, 5)
            if abilityType == "SWING" or abilityType == "SPELL" or abilityType == "RANGE" or abilityType == "SPELL_PERIODIC" then
                level = weizPVP.AbilityLevelDB[spellName]
                if level ~= nil then
                    newDataAdded = true
                else
                    --> level is nil, so make it a 0 instead
                    level = 0
                end
            end
        end

        --> If we added some new or updated info to the playerData
        if newDataAdded then
            weizPVP:UpdatePlayerDB(player, class, level, race, nil, estimated, newDataAdded, role)
        --> UpdatePlayerDB(nameAndRealm, class, level, race, guild,  estimated, dataUpdated)
        end

        return newDataAdded, role
    end
    return newDataAdded, nil
end
